@@ -7,16 +7,17 @@
 #define GPDATADIR "/usr/local/share/pari"
 #define SHELL_Q '\''
 
-#define PARIVERSION "GP/PARI CALCULATOR Version 2.15.4 (released)"
-#define PARIINFO "i386 running openbsd (ix86/GMP-%s kernel) 32-bit version"
-#define PARI_VERSION_CODE 134916
+#define PARIVERSION "GP/PARI CALCULATOR Version 2.15.5 (released)"
+#define PARIINFO "powerpc64 running openbsd (portable C/GMP-%s kernel) 64-bit version"
+#define PARI_VERSION_CODE 134917
 #define PARI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #define PARI_VERSION_SHIFT 8
 #define PARI_VCSVERSION ""
 #define PARI_MT_ENGINE "single"
 
-#define PARI_DOUBLE_FORMAT 1
-#define GCC_VERSION "OpenBSD clang version 13.0.0"
+#define PARI_DOUBLE_FORMAT -
+#define GCC_VERSION "OpenBSD clang version 16.0.6"
+#define __HAS_NO_ASM__
 #define ASMINLINE
 
 /*  Location of GNU gzip program (enables reading of .Z and .gz files). */
@@ -27,6 +28,7 @@
 #define GP_MIME_OPEN "xdg-open"
 
 #define READLINE "4.3"
+#define LONG_IS_64BIT
 #define HAS_EXP2
 #define HAS_LOG2
 #define HAS_RINT
